
#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>

#include "data.h"
#include "node.h"



node_t *node(void)
{
    return malloc(sizeof(node_t));
    // FIXME: error handling
}

node_t *leaf_node(unsigned idx)
{
    node_t *p = node();

    p->type = LEAF_NODE;
    p->l.idx = idx;
    return p;
}

node_t *branch_node(unsigned val, node_t *left, node_t *right)
{
    node_t *p = node();

    p->type = BRANCH_NODE;
    p->b.val = val;
    p->b.left = left;
    p->b.right = right;
    return p;
}


unsigned split(unsigned vl, unsigned vr)
{
    return (vl + vr)/2;
}

node_t *build_tree(pair_t *kv,
	unsigned N, unsigned P, unsigned I,
	unsigned *vl, unsigned *vr)
{
    if (P < N) {
	unsigned vll, vlr, vrl, vrr;
	node_t *l = build_tree(kv, N, 2*P, 2*I, &vll, &vlr);
	node_t *r = build_tree(kv, N, 2*P, 2*I+1, &vrl, &vrr);
	
	if (l && r) {
	    *vl = vll; *vr = vrr;
	    return branch_node(split(vlr, vrl), l, r);
	} else if (l) {
	    *vl = vll; *vr = vlr;
	    return l;
	} else if (r) {
	    *vl = vrl; *vr = vrr;
	    return r;
	} else {
	    exit(1);
	}
    } else {
	unsigned idx = I*N/P;
	if (idx == (I+1)*N/P) {
	    return NULL;
	} else {
	    *vl = *vr = kv[idx].key;
	    return leaf_node(idx);
	}
    }
}


    /* dot dump helper */

void dump_tree(node_t *node)
{
    switch (node->type) {
    case NULL_NODE:
	printf("N%p [shape=point]\n",
	    node);
	break;

    case LEAF_NODE:
	printf("N%p [label=\"%u\" shape=box]\n",
	    node, node->l.idx);
	break;

    case BRANCH_NODE:
	printf("N%p [label=\"%u\"]\n",
	    node, node->b.val);
	printf("N%p -> N%p [color=blue]\n", 
	    node, node->b.left);
	dump_tree(node->b.left);
	printf("N%p -> N%p [color=red]\n", 
	    node, node->b.right);
	dump_tree(node->b.right);
	break;
    }
}

void dump_tree_dot(node_t *node)
{
    printf("digraph BST {\n");
    printf("node [fontname=\"Arial\"]\n");
    dump_tree(node);
    printf("}\n");
}


    /* main program */

int main(void)
{
    for (int i=0; i<NUM_GEARS; i++)
	fprintf(stderr, "%u: (%d,%d)\n",
	    i, gears[i].key, gears[i].value);

    unsigned vl, vr;
    node_t *tree = build_tree(gears, NUM_GEARS, 1, 0, &vl, &vr);
    dump_tree_dot(tree);
   
    exit(0);
}
